//$$Header: $$
/*----------------------------------------------------------------------------
 This file is part of deegree, http://deegree.org/
 Copyright (C) 2001-2009 by:
   Department of Geography, University of Bonn
 and
   lat/lon GmbH

 This library is free software; you can redistribute it and/or modify it under
 the terms of the GNU Lesser General Public License as published by the Free
 Software Foundation; either version 2.1 of the License, or (at your option)
 any later version.
 This library is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 details.
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

 Contact information:

 lat/lon GmbH
 Aennchenstr. 19, 53177 Bonn
 Germany
 http://lat-lon.de/

 Department of Geography, University of Bonn
 Prof. Dr. Klaus Greve
 Postfach 1147, 53001 Bonn
 Germany
 http://www.geographie.uni-bonn.de/deegree/

 e-mail: info@deegree.org
----------------------------------------------------------------------------*/
package org.deegree.enterprise.servlet;

import java.io.IOException;
import java.io.StringReader;
import java.net.URLDecoder;
import java.util.UUID;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.deegree.framework.log.ILogger;
import org.deegree.framework.log.LoggerFactory;
import org.deegree.framework.util.StringTools;
import org.deegree.framework.xml.XMLParsingException;
import org.deegree.framework.xml.XMLTools;
import org.deegree.portal.owswatch.CommonNamepspaces;
import org.deegree.portal.owswatch.ConfigurationsException;
import org.deegree.portal.owswatch.Constants;
import org.deegree.portal.owswatch.JSPagesReference;
import org.deegree.portal.owswatch.Messages;
import org.deegree.portal.owswatch.ServiceConfiguration;
import org.deegree.portal.owswatch.ServiceLog;
import org.deegree.portal.owswatch.ServiceWatcher;
import org.deegree.portal.owswatch.ServiceWatcherFactory;
import org.deegree.portal.owswatch.ServicesConfigurationFactory;
import org.deegree.portal.owswatch.ServicesConfigurationWriter;
import org.deegree.portal.owswatch.configs.OwsWatchConfig;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.InputSource;

/**
 * The owsWatch Servlet to handle the request of the owswatch portal
 *
 * @author <a href="mailto:ncho@lat-lon.de">ncho</a>
 * @author last edited by: $Author: elmasry $
 *
 * @version $Revision: 1.3 $, $Date: 2008-03-20 16:33:27 $
 */
public class OWSWatch extends HttpServlet {

    /**
     *
     */
    private static final long serialVersionUID = -3136555273953816219L;

    private final String SESSIONID_KEY = Constants.SESSIONID_KEY;

    private static final ILogger LOG = LoggerFactory.getLogger( OWSWatch.class );

    private ServiceWatcher watcher = null;

    private ServicesConfigurationWriter servicesWriter = null;

    private String webinfPath = null;

    private String confFilePath = null;

    private ServiceWatcherFactory factory = null;

    private OwsWatchConfig conf = null;

    /*
     * (non-Javadoc)
     *
     * @see javax.servlet.GenericServlet#init()
     */
    @Override
    public void init()
                            throws ServletException {

        confFilePath = this.getServletContext().getRealPath( this.getInitParameter( "owsWatchConfiguration" ) );
        webinfPath = this.getServletContext().getRealPath( "WEB-INF/conf/owswatch" );
        if ( !webinfPath.endsWith( "/" ) ) {
            webinfPath = webinfPath.concat( "/" );
        }

        try {
            factory = ServiceWatcherFactory.getInstance( confFilePath, webinfPath );
            watcher = factory.getServiceWatcherInstance();
            conf = factory.getConf();

        } catch ( ConfigurationsException e ) {
            LOG.logError( e.getLocalizedMessage() );
        }
        // An error has occured parsing the configurations file
        if ( factory == null || conf == null ) {
            LOG.logError( "There seems to be a problem with your configurations file. owsWatch will not start" );
            return;
        }

        watcher.compileDownTimeReport( webinfPath, conf );
        watcher.start();
    }

    /*
     * (non-Javadoc)
     *
     * @see javax.servlet.http.HttpServlet#doPost(javax.servlet.http.HttpServletRequest,
     *      javax.servlet.http.HttpServletResponse)
     */
    @Override
    public void doPost( HttpServletRequest request, HttpServletResponse response ) {
        performAction( request, response );
    }

    /*
     * (non-Javadoc)
     *
     * @see javax.servlet.http.HttpServlet#doGet(javax.servlet.http.HttpServletRequest,
     *      javax.servlet.http.HttpServletResponse)
     */
    @Override
    public void doGet( HttpServletRequest request, HttpServletResponse response ) {
        performAction( request, response );
    }

    /**
     * determines which action is to be executed and calls the corresponding method of that action: LOGIN - send a
     * REQUEST - read a PROTOCOL
     *
     * @param request
     * @param response
     * @return boolean if everything went fine. false otherwise. This class will handle the Errors
     */
    private boolean performAction( HttpServletRequest request, HttpServletResponse response ) {

        if ( request == null ) {
            gotoErrorPage( request, response, Messages.getMessage( "ERROR_NULL_OBJ", "HttpServletRequest" ), null, null );
            return false;
        }

        if ( response == null ) {
            gotoErrorPage( request, response, Messages.getMessage( "ERROR_NULL_OBJ", "HttpServletResponse" ), null,
                           null );
            return false;
        }

        String action = request.getParameter( "action" );

        LOG.logDebug( "The action is: ", action );

        if ( watcher == null || ( !isLoggedIn( request ) && !"LOGIN".equals( action ) ) ) {
            return handleLogout( request, response );
        }

        if ( action != null ) {
            if ( action.equals( "LOGIN" ) ) {
                handleLogin( request, response );
            } else if ( action.equals( "LOGOUT" ) ) {
                handleLogout( request, response );
            } else if ( action.equals( "stopServiceMonitor" ) ) {
                stopServiceMonitor( request, response );
            } else if ( action.equals( "startServiceMonitor" ) ) {
                startServiceMonitor( request, response );
            } else if ( action.equals( "logout" ) ) {
                handleLogout( request, response );
            } else if ( action.equals( "gotoLogin" ) ) {
                gotoLoginPage( request, response );
            } else if ( action.equals( "serviceDelete" ) ) {
                handleServiceDelete( request, response );
            } else if ( action.equals( "serviceEdit" ) ) {
                handleServiceEdit( request, response );
            } else if ( action.equals( "serviceTest" ) ) {
                handleServiceTest( request, response );
            } else if ( action.equals( "refreshPage" ) ) {
                handleRefreshPage( request, response );
            } else if ( action.equals( "cancelServiceManager" ) ) {
                handleRefreshPage( request, response );
            } else if ( action.equals( "addService" ) ) {
                handleAddService( request, response );
            } else {
                gotoErrorPage( request, response, StringTools.concat( 100,
                                                                      Messages.getMessage( "ERROR_PARAM_UNEXPECTED",
                                                                                           "action", action ) ), null,
                               null );
                return false;
            }
        } else {
            // action is null. The request has to be an rpc
            String requestName = request.getParameter( "rpc" );
            if ( requestName != null ) {
                return handleRPCRequest( request, response, requestName );
            }
            return true;
        }
        return false;
    }

    /**
     * Takes in a POST request and directs it to the corresponding function
     *
     * @param requestValue
     *            String sent from the POST form
     */
    private boolean handleRPCRequest( HttpServletRequest request, HttpServletResponse response, String requestValue ) {

        Document doc = null;
        try {
            doc = parseDocument( requestValue );
        } catch ( XMLParsingException e ) {
            gotoErrorPage( request, response, e.getLocalizedMessage(), "Go Back to the main page",
                           JSPagesReference.getString( "OWSWatch.owswatchMonitorList" ) );
            return false;
        }
        if ( doc == null ) {
            gotoErrorPage( request, response, "The RPC request is null", Messages.getString( "MESSAGE_GOTO_MAIN" ),
                           JSPagesReference.getString( "OWSWatch.owswatchMonitorList" ) );
            return false;
        }
        Element root = doc.getDocumentElement();
        String rpcType = root.getAttribute( "type" );
        if ( "SaveService".equals( rpcType ) ) {
            if ( handleSaveService( request, response, root ) ) {
                return handleRefreshPage( request, response );
            } else {
                return false;
            }
        } else if ( "SaveConfigs".equals( rpcType ) ) {
            // TODO will be used later to edit the serviceconfig.xml
            return true;
        } else {
            return true;
        }
    }

    /**
     * Takes in a addService Request and directs to the corresponding page
     *
     * @return boolean if no errors were thrown, false otherwise
     */
    private boolean handleAddService( HttpServletRequest request, HttpServletResponse response ) {

        storeSelectedServiceInSession( request );
        HttpSession session = request.getSession( true );
        // Indicates this is a new service
        session.setAttribute( "EditService", Boolean.valueOf( false ) );
        String[] ns = { factory.getServicesParser().getPrefix(), CommonNamepspaces.DEEGREEWSNS.toASCIIString() };
        session.setAttribute( "PREFIX_NS", ns );
        session.setAttribute( "ServiceConfigs", watcher.getServices() );
        String nextpage = JSPagesReference.getString( "OWSWatch.editTest" );
        try {
            response.sendRedirect( nextpage );
        } catch ( Exception e ) {
            gotoErrorPage( request, response, Messages.getMessage( "ERROR_PAGE_NOT_FOUND", nextpage ),
                           Messages.getString( "MESSAGE_GOTO_MAIN" ),
                           JSPagesReference.getString( "OWSWatch.owswatchMonitorList" ) );
            return false;
        }
        return true;
    }

    /**
     * handles SaveService requests whether its a new service or edited service
     *
     * @param rpcRequest
     * @return boolean if no errors happened
     */
    private boolean handleSaveService( HttpServletRequest request, HttpServletResponse response, Element rpcRequest ) {

        ServicesConfigurationFactory parser = factory.getServicesParser();
        ServiceConfiguration service = null;
        try {
            String xPath = StringTools.concat( 100, "./ServiceXML/", factory.getServicesParser().getPrefix(),
                                               ":SERVICE" );
            Element serviceElem = XMLTools.getElement( rpcRequest, xPath, ServicesConfigurationFactory.getCnxt() );
            service = parser.parseService( serviceElem, factory.getServicesParser().getPrefix() );
        } catch ( Exception e ) {
            String errorMsg = Messages.arrayToString( new String[] { Messages.getString( "ERROR_SAVE_SERVICE" ),
                                                                    e.getLocalizedMessage() }, "\n" );
            gotoErrorPage( request, response, errorMsg, Messages.getString( "MESSAGE_GOTO_MAIN" ),
                           JSPagesReference.getString( "OWSWatch.owswatchMonitorList" ) );
            return false;
        }

        ServiceLog serviceLog = null;
        try {
            if ( watcher.getServices().containsKey( service.getServiceid() ) ) {
                serviceLog = watcher.getServiceLogs().get( service.getServiceid() );
            } else {

                serviceLog = new ServiceLog( factory.getProtDirPath(), service.getServiceid(),
                                             service.getServiceName(), service.getServiceType(),
                                             factory.getServletAddr(), factory.getSender() );
            }
            watcher.addService( service, serviceLog );
            servicesWriter.writeDocument( watcher.getServices() );
        } catch ( Exception e ) {
            LOG.logError( e.getMessage(), e );
            gotoErrorPage( request, response, e.getMessage(), Messages.getMessage( "MESSAGE_GOTO_MAIN" ),
                           JSPagesReference.getString( "OWSWatch.login" ) );
            return false;
        }

        return true;
    }

    /**
     * handles the administrator login
     *
     * @return true if no errors happened, false otherwise
     */
    private boolean handleLogin( HttpServletRequest request, HttpServletResponse response ) {

        // If the configuration files could not be parsed correctly, the program should not proceed
        if ( factory == null || conf == null ) {
            gotoErrorPage( request, response, Messages.getMessage( "INCORRECT_LOGIN" ), null, null );
        }

        String user = request.getParameter( "username" );
        String pwd = request.getParameter( "password" );

        try {
            if ( conf.isAuthenticatedUser( user, pwd ) ) {
                servicesWriter = new ServicesConfigurationWriter( webinfPath
                                                                  + conf.getGeneral().getServiceInstancesPath(),
                                                                  factory.getServicesParser().getPrefix() );
                HttpSession session = request.getSession( true );
                session.setAttribute( "Services", this.watcher.getServices() );
                session.setAttribute( "Logs", watcher.getServiceLogs() );
                session.setAttribute( "GLOBAL_REFRESH", conf.getGeneral().getGlobalRefreshRate() );
                session.setAttribute( "ThreadSuspended", false );
                session.setAttribute( "ServiceDescription", conf.getServiceConfig() );
                // isLoggedin
                String sessionId = UUID.randomUUID().toString();
                session.setAttribute( SESSIONID_KEY, sessionId );
                session.setAttribute( "isLoggedin", true );
                String nextpage = JSPagesReference.getString( "OWSWatch.owswatchMonitorList" );
                response.sendRedirect( nextpage );
            } else {
                gotoErrorPage( request, response, Messages.getMessage( "INCORRECT_LOGIN" ),
                               Messages.getMessage( "MESSAGE_GOTO_MAIN" ),
                               JSPagesReference.getString( "OWSWatch.login" ) );
            }
        } catch ( Exception e ) {
            String errorMsg = StringTools.concat( 100, Messages.getMessage( "ERROR_LOGIN" ), "</br>",
                                                  e.getLocalizedMessage() );
            gotoErrorPage( request, response, errorMsg, Messages.getMessage( "MESSAGE_GOTO_MAIN" ),
                           JSPagesReference.getString( "OWSWatch.login" ) );
            return false;
        }
        return true;
    }

    /**
     * Stops Monitoring a certain service. i.e. stop refreshing the page after time intervals
     *
     * @param request
     * @param response
     */
    private boolean stopServiceMonitor( HttpServletRequest request, HttpServletResponse response ) {

        int index = Integer.parseInt( request.getParameter( "reqIndex" ) );
        watcher.stopServiceConfiguration( index );
        HttpSession session = request.getSession( true );
        session.setAttribute( "Services", this.watcher.getServices() );
        session.setAttribute( "Logs", watcher.getServiceLogs() );
        String nextpage = JSPagesReference.getString( "OWSWatch.owswatchMonitorList" );
        try {
            response.sendRedirect( nextpage );
        } catch ( Exception e ) {
            LOG.logError( "The ServiceConfiguration could not be stopped", e );
            return false;
        }
        return true;
    }

    /**
     * Starts monitoring a certain service. i.e. starts sending GetCapabilities requests after predefined intervals
     *
     */
    private boolean startServiceMonitor( HttpServletRequest request, HttpServletResponse response ) {

        int index = Integer.parseInt( request.getParameter( "reqIndex" ) );
        watcher.startServiceConfiguration( index );
        HttpSession session = request.getSession( true );
        session.setAttribute( "Services", this.watcher.getServices() );
        session.setAttribute( "Logs", watcher.getServiceLogs() );
        String nextpage = JSPagesReference.getString( "OWSWatch.owswatchMonitorList" );

        try {
            response.sendRedirect( nextpage );
        } catch ( Exception e ) {
            LOG.logError( "The ServiceConfiguration could not be started", e );
            return false;
        }
        return true;
    }

    /**
     * Logs the user out
     *
     * @return true if logout successfully, false otherwise
     */
    private boolean handleLogout( HttpServletRequest request, HttpServletResponse response ) {

        HttpSession session = request.getSession( true );
        session.setAttribute( "isLoggedin", false );
        session.removeAttribute( SESSIONID_KEY );

        String nextPage = JSPagesReference.getString( "OWSWatch.login" );
        try {
            response.sendRedirect( nextPage );
        } catch ( Exception e ) {
            gotoErrorPage( request, response, Messages.getMessage( "ERROR_PAGE_NOT_FOUND", nextPage ), null, null );
            return false;
        }

        return true;
    }

    /**
     * goto the first page
     *
     * @return true if no error happened, false otherwise
     */
    private boolean gotoLoginPage( HttpServletRequest request, HttpServletResponse response ) {

        String nextPage = JSPagesReference.getString( "OWSWatch.owswatchMonitorList" );
        try {
            response.sendRedirect( nextPage );
        } catch ( Exception e ) {
            gotoErrorPage( request, response, Messages.getMessage( "ERROR_PAGE_NOT_FOUND", nextPage ), null, null );
            return false;
        }

        return true;
    }

    /**
     * deletes a service
     *
     * @return true if no errors happened, false otherwise
     */
    private boolean handleServiceDelete( HttpServletRequest request, HttpServletResponse response ) {

        storeSelectedServiceInSession( request );
        int serviceId = Integer.valueOf( request.getParameter( "serviceId" ) );
        try {
            watcher.removeService( serviceId );
            servicesWriter.writeDocument( watcher.getServices() );
        } catch ( Exception e ) {
            gotoErrorPage( request, response, e.getMessage(), Messages.getMessage( "MESSAGE_GOTO_MAIN" ),
                           JSPagesReference.getString( "OWSWatch.owswatchMonitorList" ) );
        }
        HttpSession session = request.getSession( true );
        session.setAttribute( "Services", watcher.getServices() );
        session.setAttribute( "Logs", watcher.getServiceLogs() );
        String nextpage = JSPagesReference.getString( "OWSWatch.owswatchMonitorList" );

        try {
            response.sendRedirect( nextpage );
        } catch ( Exception e ) {
            gotoErrorPage( request, response, Messages.getMessage( "ERROR_PAGE_NOT_FOUND", nextpage ),
                           Messages.getMessage( "MESSAGE_GOTO_MAIN" ),
                           JSPagesReference.getString( "OWSWatch.owswatchMonitorList" ) );
            return false;
        }
        return true;
    }

    /**
     * Redirects the reuqest to the ServiceManager Dialogue
     *
     * @return ture if no errors happened, false otherwise
     */
    private boolean handleServiceEdit( HttpServletRequest request, HttpServletResponse response ) {

        storeSelectedServiceInSession( request );
        HttpSession session = request.getSession( true );
        // Indicates this is an existing service
        session.setAttribute( "EditService", Boolean.valueOf( true ) );
        ServiceConfiguration service = null;
        try {
            int serviceId = Integer.parseInt( request.getParameter( "serviceId" ) );
            service = watcher.getService( serviceId );
        } catch ( Exception e ) {
            gotoErrorPage( request, response, Messages.getMessage( "ERROR_MISSING_KEY", "ServiceId" ),
                           Messages.getMessage( "MESSAGE_GOTO_MAIN" ),
                           JSPagesReference.getString( "OWSWatch.owswatchMonitorList" ) );
            return false;
        }

        String[] ns = { factory.getServicesParser().getPrefix(), CommonNamepspaces.DEEGREEWSNS.toASCIIString() };
        session.setAttribute( "PREFIX_NS", ns );
        // The service to edit in the jsp page
        session.setAttribute( "ServiceToEdit", service );
        String nextpage = JSPagesReference.getString( "OWSWatch.editTest" );

        try {
            response.sendRedirect( nextpage );
        } catch ( Exception e ) {
            gotoErrorPage( request, response, Messages.getMessage( "ERROR_PAGE_NOT_FOUND", nextpage ),
                           Messages.getMessage( "MESSAGE_GOTO_MAIN" ),
                           JSPagesReference.getString( "OWSWatch.owswatchMonitorList" ) );
            return false;
        }
        return true;
    }

    /**
     * Executes the given test once
     *
     * @return true if no errors happened, false otherwise
     */
    private boolean handleServiceTest( HttpServletRequest request, HttpServletResponse response ) {

        storeSelectedServiceInSession( request );
        String Id = request.getParameter( "serviceId" );
        watcher.execute( Integer.valueOf( Id ) );
        HttpSession session = request.getSession( true );
        session.setAttribute( "Services", watcher.getServices() );
        session.setAttribute( "Logs", watcher.getServiceLogs() );
        String nextpage = JSPagesReference.getString( "OWSWatch.owswatchMonitorList" );

        try {
            response.sendRedirect( nextpage );
        } catch ( Exception e ) {
            gotoErrorPage( request, response, Messages.getMessage( "ERROR_PAGE_NOT_FOUND", nextpage ),
                           Messages.getMessage( "MESSAGE_GOTO_MAIN" ),
                           JSPagesReference.getString( "OWSWatch.owswatchMonitorList" ) );
            return false;
        }
        return true;
    }

    /**
     * Gets the selectedservice(case exists) Id and stores in the session
     *
     */
    private void storeSelectedServiceInSession( HttpServletRequest request ) {
        HttpSession session = request.getSession( true );
        String tmp = (String) request.getParameter( "selectedService" );
        if ( tmp != null & session != null ) {
            try {
                session.setAttribute( "selectedService", new Integer( tmp ) );
            } catch ( Exception e ) {
                session.setAttribute( "selectedService", new Integer( -1 ) );
            }
        }
    }

    private void gotoErrorPage( HttpServletRequest request, HttpServletResponse response, String error, String urlText,
                                String url ) {

        LOG.logError( "The error occured is: " + error );
        HttpSession session = request.getSession( true );

        session.setAttribute( "message", StringTools.replace( error, "\n", "<br/>", true ) );
        if ( error == null ) {
            error = "An unknown error has occured";
        }
        if ( urlText == null ) {
            urlText = "";
        }
        session.setAttribute( "URLText", urlText );
        if ( url == null ) {
            url = "";
        }
        session.setAttribute( "URLAdd", url );
        try {
            String nextpage = JSPagesReference.getString( "OWSWatch.error" );
            response.sendRedirect( nextpage );
        } catch ( Exception e ) {
            LOG.logError( "The page could not be redirected to the error page" );
        }
    }

    /**
     * goto the main Monitor list
     *
     * @return true if no errors happened, false otherwise
     */
    private boolean handleRefreshPage( HttpServletRequest request, HttpServletResponse response ) {

        storeSelectedServiceInSession( request );
        HttpSession session = request.getSession( true );
        String nextpage = null;
        if ( watcher != null ) {
            session.setAttribute( "Services", watcher.getServices() );
            session.setAttribute( "Logs", watcher.getServiceLogs() );
            nextpage = JSPagesReference.getString( "OWSWatch.owswatchMonitorList" );
        }

        try {
            response.sendRedirect( nextpage );
        } catch ( Exception e ) {
            gotoErrorPage( request, response, Messages.getMessage( "ERROR_PAGE_NOT_FOUND", nextpage ), null, null );
            return false;
        }
        return true;
    }

    /**
     * Creates a new instance of DocumentBuilder
     *
     * @return DocumentBuilder
     * @throws IOException
     */
    private DocumentBuilder instantiateParser()
                            throws IOException {

        DocumentBuilder parser = null;

        try {
            DocumentBuilderFactory fac = DocumentBuilderFactory.newInstance();
            fac.setNamespaceAware( true );
            fac.setValidating( false );
            fac.setIgnoringElementContentWhitespace( true );
            parser = fac.newDocumentBuilder();
            return parser;
        } catch ( ParserConfigurationException e ) {
            throw new IOException( "Unable to initialize DocumentBuilder: " + e.getMessage() );
        }
    }

    /**
     * Converts a given xml string to a document
     *
     * @param xmlText
     * @return Document
     */
    private Document parseDocument( String xmlText )
                            throws XMLParsingException {

        Document doc = null;
        try {
            String dec = URLDecoder.decode( xmlText, "UTF-8" );
            doc = instantiateParser().parse( new InputSource( new StringReader( dec ) ) );
        } catch ( Exception e ) {
            throw new XMLParsingException( "Error parsing xml document\n" + e.getLocalizedMessage() );
        }
        return doc;
    }

    /**
     * Verifies that this user is loggedin through comparing the session ID from the request paarameter with that saved
     * in the session
     *
     * @param request
     * @return true if the user is loggedin, false otherwise
     */
    protected boolean isLoggedIn( HttpServletRequest request ) {
        HttpSession session = request.getSession( true );
        String requestSession = request.getParameter( SESSIONID_KEY );
        String sessionId = (String) session.getAttribute( SESSIONID_KEY );
        if ( requestSession == null || sessionId == null || !requestSession.equals( sessionId ) ) {
            return false;
        }
        return true;
    }
}
