//$HeadURL: https://svn.wald.intevation.org/svn/deegree/base/branches/2.3_testing/src/org/deegree/graphics/Selector.java $
/*----------------------------------------------------------------------------
 This file is part of deegree, http://deegree.org/
 Copyright (C) 2001-2009 by:
   Department of Geography, University of Bonn
 and
   lat/lon GmbH

 This library is free software; you can redistribute it and/or modify it under
 the terms of the GNU Lesser General Public License as published by the Free
 Software Foundation; either version 2.1 of the License, or (at your option)
 any later version.
 This library is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 details.
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

 Contact information:

 lat/lon GmbH
 Aennchenstr. 19, 53177 Bonn
 Germany
 http://lat-lon.de/

 Department of Geography, University of Bonn
 Prof. Dr. Klaus Greve
 Postfach 1147, 53001 Bonn
 Germany
 http://www.geographie.uni-bonn.de/deegree/

 e-mail: info@deegree.org
----------------------------------------------------------------------------*/

package org.deegree.graphics;

import org.deegree.model.spatialschema.Envelope;
import org.deegree.model.spatialschema.Position;

/**
 *
 * <p>
 * ------------------------------------------------------------------------
 * </p>
 *
 * @author <a href="mailto:poth@lat-lon.de">Andreas Poth</a>
 * @version $Revision: 18195 $ $Date: 2009-06-18 17:55:39 +0200 (Do, 18. Jun 2009) $
 */
public interface Selector {

    /**
     * adds a Theme to the Selector that shall be notified if something happens.
     * @param theme to add.
     */
    void addTheme( Theme theme );

    /**
     * @param theme to be removed
     * @see Selector#addTheme(Theme)
     */
    void removeTheme( Theme theme );

    /**
     * selects all features (display elements) that are located within the submitted bounding box.
     * @param boundingbox to select all features within
     *
     * @return id's of the selected features (display elements)
     */
    String[] select( Envelope boundingbox );

    /**
     * selects all features (display elements) that intersects the submitted point. if a feature is
     * already selected it remains selected.
     * @param position to select the features with.
     *
     * @return id's of the selected features (display elements)
     */
    String[] select( Position position );

    /**
     * selects all features (display elements) that are located within the circle described by the
     * position and the radius. if a feature is already selected it remains selected.
     * @param position to select al features
     * @param radius around the given position
     *
     * @return id's of the selected features (display elements)
     */
    String[] select( Position position, double radius );

    /**
     * selects all features (display elements) that are specified by the submitted ids. if a feature
     * is already selected it remains selected.
     * @param ids to select
     *
     * @return ids of the selected features (display elements)
     */
    String[] select( String[] ids );

    /**
     * inverts the current selection.
     * @return the id's of the inverted selection
     */
    String[] invertSelection();

    /**
     * mark all features as unselected
     */
    void reset();

}
