//$HeadURL: https://svn.wald.intevation.org/svn/deegree/base/branches/2.3_testing/src/org/deegree/ogcwebservices/sos/capabilities/SOSGetCapabilities.java $
/*----------------------------------------------------------------------------
 This file is part of deegree, http://deegree.org/
 Copyright (C) 2001-2009 by:
   Department of Geography, University of Bonn
 and
   lat/lon GmbH

 This library is free software; you can redistribute it and/or modify it under
 the terms of the GNU Lesser General Public License as published by the Free
 Software Foundation; either version 2.1 of the License, or (at your option)
 any later version.
 This library is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 details.
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

 Contact information:

 lat/lon GmbH
 Aennchenstr. 19, 53177 Bonn
 Germany
 http://lat-lon.de/

 Department of Geography, University of Bonn
 Prof. Dr. Klaus Greve
 Postfach 1147, 53001 Bonn
 Germany
 http://www.geographie.uni-bonn.de/deegree/

 e-mail: info@deegree.org
----------------------------------------------------------------------------*/
package org.deegree.ogcwebservices.sos.capabilities;

import java.util.HashMap;
import java.util.Map;

import org.deegree.framework.util.StringTools;
import org.deegree.framework.xml.NamespaceContext;
import org.deegree.framework.xml.XMLParsingException;
import org.deegree.framework.xml.XMLTools;
import org.deegree.ogcbase.CommonNamespaces;
import org.deegree.ogcwebservices.OGCWebServiceException;
import org.deegree.ogcwebservices.getcapabilities.GetCapabilities;
import org.w3c.dom.Document;

/**
 * represents a sOs getCapabilities Request
 *
 * @author <a href="mailto:mkulbe@lat-lon.de">Matthias Kulbe </a>
 * @author last edited by: $Author:wanhoff$
 *
 * @version $Revision: 18195 $, $Date:20.03.2007$
 */
public class SOSGetCapabilities extends GetCapabilities {

    private static final long serialVersionUID = 6229665412183042265L;

    private static final NamespaceContext nsContext = CommonNamespaces.getNamespaceContext();

    /**
     * creates a GetCapabilities Request from a KVP Map
     *
     * @param map
     * @return the new instance
     */
    public static SOSGetCapabilities create( Map<String, String> map ) {

        String id = getParam( "ID", map, "" + System.currentTimeMillis() );

        // optional
        // String version = getParam( "VERSION", map, null );

        // optional
        String updateSequence = getParam( "UPDATESEQUENCE", map, null );

        // optional and unbounded
        String[] sections = null;
        if ( map.get( "SECTIONS" ) != null ) {
            String tmp = getParam( "SECTIONS", map, null );
            sections = StringTools.toArray( tmp, ",", false );
        }

        // optional and unbounded
        String[] acceptVersions = null;
        if ( map.get( "ACCEPTVERSIONS" ) != null ) {
            String tmp = getParam( "ACCEPTVERSIONS", map, null );
            acceptVersions = StringTools.toArray( tmp, ",", false );
        }

        // optional and unbounded
        String[] acceptFormats = null;
        if ( map.get( "ACCEPTFORMATS" ) != null ) {
            String tmp = getParam( "ACCEPTFORMATS", map, null );
            acceptFormats = StringTools.toArray( tmp, ",", false );
        }

        return new SOSGetCapabilities( id, updateSequence, acceptVersions, sections, acceptFormats, map );

    }

    /**
     * creates GetCapabilities Request from XML
     *
     * @param id
     * @param doc
     * @return the new instance
     * @throws OGCWebServiceException
     *
     */
    public static SOSGetCapabilities create( String id, Document doc )
                            throws OGCWebServiceException {

        try {
            // optional
            // String version = XMLTools.getNodeAsString( doc, "ows:GetCapabilities/@version",
            // nsContext,
            // null );

            // optional
            String updateSequence = XMLTools.getNodeAsString( doc, "ows:GetCapabilities/@updateSequence", nsContext,
                                                              null );

            // optional and unbounded
            String[] sections = XMLTools.getNodesAsStrings( doc, "ows:GetCapabilities/ows:Sections/ows:Section/text()",
                                                            nsContext );

            // optional and unbounded
            String[] acceptVersions = XMLTools.getNodesAsStrings(
                                                                  doc,
                                                                  "ows:GetCapabilities/ows:AcceptVersions/ows:Version/text()",
                                                                  nsContext );

            // optional and unbounded
            String[] acceptFormats = XMLTools.getNodesAsStrings(
                                                                 doc,
                                                                 "ows:GetCapabilities/ows:AcceptFormats/ows:OutputFormat/text()",
                                                                 nsContext );

            return new SOSGetCapabilities( id, updateSequence, acceptVersions, sections, acceptFormats,
                                           new HashMap<String, String>() );

        } catch ( XMLParsingException e ) {
            e.printStackTrace();
            throw new OGCWebServiceException( "sos webservice failure" );
        }
    }

    /**
     * creates GetCapabilities Request
     *
     * @param sections
     * @param acceptFormats
     * @param acceptVersions
     * @param id
     * @param updateSequence
     * @param vendoreSpec
     * @return the new instance
     *
     */
    public static SOSGetCapabilities create( String id, String[] sections, String[] acceptFormats,
                                             String[] acceptVersions, String updateSequence,
                                             Map<String, String> vendoreSpec ) {
        return new SOSGetCapabilities( id, updateSequence, acceptVersions, sections, acceptFormats, vendoreSpec );
    }

    /**
     * @param id
     * @param updateSequence
     * @param acceptVersions
     * @param sections
     * @param acceptFormats
     * @param vendoreSpec
     */
    public SOSGetCapabilities( String id, String updateSequence, String[] acceptVersions, String[] sections,
                               String[] acceptFormats, Map<String, String> vendoreSpec ) {
        super( id, null, updateSequence, acceptVersions, sections, acceptFormats, vendoreSpec );
    }

    /**
     * fixed 'SOS'
     *
     * @return the String "SOS"
     */
    public String getServiceName() {
        return "SOS";
    }
}
