//$HeadURL: https://svn.wald.intevation.org/svn/deegree/base/branches/2.3_testing/src/org/deegree/ogcwebservices/wms/configuration/WMSConfigurationType.java $
/*----------------------------------------------------------------------------
 This file is part of deegree, http://deegree.org/
 Copyright (C) 2001-2009 by:
   Department of Geography, University of Bonn
 and
   lat/lon GmbH

 This library is free software; you can redistribute it and/or modify it under
 the terms of the GNU Lesser General Public License as published by the Free
 Software Foundation; either version 2.1 of the License, or (at your option)
 any later version.
 This library is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 details.
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

 Contact information:

 lat/lon GmbH
 Aennchenstr. 19, 53177 Bonn
 Germany
 http://lat-lon.de/

 Department of Geography, University of Bonn
 Prof. Dr. Klaus Greve
 Postfach 1147, 53001 Bonn
 Germany
 http://www.geographie.uni-bonn.de/deegree/

 e-mail: info@deegree.org
----------------------------------------------------------------------------*/
package org.deegree.ogcwebservices.wms.configuration;

import java.net.URL;
import java.util.List;

import org.deegree.ogcwebservices.wms.capabilities.Layer;
import org.deegree.owscommon_new.OperationsMetadata;
import org.deegree.owscommon_new.ServiceIdentification;

/**
 * <code>WMSConfigurationType</code> defines the methods that each WMS configuration object has to
 * implement. It is used to unify the implementations for the different versions.
 *
 * @author <a href="mailto:schmitz@lat-lon.de">Andreas Schmitz</a>
 * @author last edited by: $Author: mschneider $
 *
 * @version 2.0, $Revision: 18195 $, $Date: 2009-06-18 17:55:39 +0200 (Do, 18. Jun 2009) $
 *
 * @since 2.0
 */

public interface WMSConfigurationType {

    /**
     * @return Returns the deegreeParams.
     */
    public WMSDeegreeParams getDeegreeParams();

    /**
     * @param deegreeParams
     *            The deegreeParams to set.
     */
    public void setDeegreeParams( WMSDeegreeParams deegreeParams );

    /**
     * @return Gets the base URL which is used to resolve file resource (XSL sheets).
     */
    public URL getBaseURL();

    /**
     * @return the version
     */
    public String getVersion();

    /**
     * @return the <code>ServiceIdentification</code> object
     */
    public ServiceIdentification getServiceIdentification();

    /**
     * @return the updateSequence.
     */
    public String getUpdateSequence();

    /**
     *
     * @return the root layer provided by a WMS
     */
    public Layer getLayer();

    /**
     *
     * @param name
     *            the layer name
     * @return the layer provided by a WMS or null, if there is no such layer
     */
    public Layer getLayer( String name );

    /**
     * @return the operations metadata object
     */
    public OperationsMetadata getOperationMetadata();

    /**
     *
     * @param version
     *            the input version
     * @return the nearest supported version according to the version negotiation rules.
     */
    public String calculateVersion( String version );

    /**
     * @return the list of available exceptions
     */
    public List<String> getExceptions();

}
