/*----------------------------------------------------------------------------
 This file is part of deegree, http://deegree.org/
 Copyright (C) 2001-2009 by:
   Department of Geography, University of Bonn
 and
   lat/lon GmbH

 This library is free software; you can redistribute it and/or modify it under
 the terms of the GNU Lesser General Public License as published by the Free
 Software Foundation; either version 2.1 of the License, or (at your option)
 any later version.
 This library is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 details.
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

 Contact information:

 lat/lon GmbH
 Aennchenstr. 19, 53177 Bonn
 Germany
 http://lat-lon.de/

 Department of Geography, University of Bonn
 Prof. Dr. Klaus Greve
 Postfach 1147, 53001 Bonn
 Germany
 http://www.geographie.uni-bonn.de/deegree/

 e-mail: info@deegree.org
----------------------------------------------------------------------------*/
package org.deegree.ogcwebservices.wpvs;

import org.deegree.ogcwebservices.OGCWebServiceException;
import org.deegree.ogcwebservices.wpvs.operation.Get3DFeatureInfo;
import org.deegree.ogcwebservices.wpvs.operation.Get3DFeatureInfoResponse;

/**
 *
 * @version $Revision: $
 * @author <a href="mailto:cordes@lat-lon.de">Lyn Buesching</a>
 * @author last edited by: $Author: $
 *
 * @version 1.0. $Revision: $, $Date: $
 *
 */
public abstract class Get3DFeatureInfoHandler {

    private WPVService owner;

    /**
     * @param owner
     *            of this infohandler, actually the service instance.
     */
    public Get3DFeatureInfoHandler( WPVService owner ) {
        this.owner = owner;
    }

    /**
     * sets the owner to <code>null</code>
     */
    public Get3DFeatureInfoHandler() {
        this.owner = null;
    }

    /**
     * Handle the Get3DfeatureInfo request given by <code>Get3DFeatureInfoRequest</code>
     *
     * @param request
     * @return an instance of Get3DFeatureInfoResponse
     * @throws OGCWebServiceException
     *             if an error occurred while handling the request.
     */
    public abstract Get3DFeatureInfoResponse handleRequest( Get3DFeatureInfo request )
                            throws OGCWebServiceException;

    /**
     *
     * @return the owner or <code>null</code> if it was not set
     */
    protected final WPVService getOwner() {
        return owner;
    }
}
