//$HeadURL: https://svn.wald.intevation.org/svn/deegree/base/branches/2.3_testing/src/org/deegree/portal/context/AbstractFrontend.java $
/*----------------------------------------------------------------------------
 This file is part of deegree, http://deegree.org/
 Copyright (C) 2001-2009 by:
   Department of Geography, University of Bonn
 and
   lat/lon GmbH

 This library is free software; you can redistribute it and/or modify it under
 the terms of the GNU Lesser General Public License as published by the Free
 Software Foundation; either version 2.1 of the License, or (at your option)
 any later version.
 This library is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 details.
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

 Contact information:

 lat/lon GmbH
 Aennchenstr. 19, 53177 Bonn
 Germany
 http://lat-lon.de/

 Department of Geography, University of Bonn
 Prof. Dr. Klaus Greve
 Postfach 1147, 53001 Bonn
 Germany
 http://www.geographie.uni-bonn.de/deegree/

 e-mail: info@deegree.org
----------------------------------------------------------------------------*/
package org.deegree.portal.context;

import java.util.ArrayList;
import java.util.List;

/**
 * this class encapsulates the description of the front end of a GUI setting up on a web map
 * context. this is a deegree specific form of description. beside some
 *
 * @author <a href="mailto:poth@lat-lon.de">Andreas Poth</a>
 * @author last edited by: $Author: mschneider $
 *
 * @version $Revision: 18195 $, $Date: 2009-06-18 17:55:39 +0200 (Do, 18. Jun 2009) $
 */
public abstract class AbstractFrontend implements Frontend {

    private GUIArea center = null;

    private GUIArea east = null;

    private GUIArea north = null;

    private GUIArea south = null;

    private GUIArea west = null;

    private String controller = null;

    /**
     * Creates a new Frontend object.
     *
     * @param controller
     * @param west
     * @param east
     * @param south
     * @param north
     * @param center
     */
    public AbstractFrontend( String controller, GUIArea west, GUIArea east, GUIArea south, GUIArea north, GUIArea center ) {
        setController( controller );
        setWest( west );
        setEast( east );
        setSouth( south );
        setNorth( north );
        setCenter( center );
    }

    /**
     * @return the name of the central controller of the front end. depending on the implementation
     *         this may be the name of a HTML/JSP-page, a java class or something else.
     */
    public String getController() {
        return controller;
    }

    /**
     * @return the description of the west GUI area
     */
    public GUIArea getWest() {
        return west;
    }

    /**
     * @return the description of the east GUI area
     */
    public GUIArea getEast() {
        return east;
    }

    /**
     * @return the description of the south GUI area
     */
    public GUIArea getSouth() {
        return south;
    }

    /**
     * @return the description of the north GUI area
     */
    public GUIArea getNorth() {
        return north;
    }

    /**
     * @return the description of the central GUI area
     */
    public GUIArea getCenter() {
        return center;
    }

    /**
     * sets the name of the central controller of the front end. depending on the implementation
     * this may be the name of a HTML/JSP-page a java class or something else.
     *
     * @param controller
     */
    public void setController( String controller ) {
        this.controller = controller;
    }

    /**
     * sets the description of the west GUI area
     *
     * @param west
     */
    public void setWest( GUIArea west ) {
        this.west = west;
    }

    /**
     * sets the description of the east GUI area
     *
     * @param east
     */
    public void setEast( GUIArea east ) {
        this.east = east;
    }

    /**
     * sets the description of the south GUI area
     *
     * @param south
     */
    public void setSouth( GUIArea south ) {
        this.south = south;
    }

    /**
     * sets the description of the north GUI area
     *
     * @param north
     */
    public void setNorth( GUIArea north ) {
        this.north = north;
    }

    /**
     * sets the description of the central GUI area
     *
     * @param center
     */
    public void setCenter( GUIArea center ) {
        this.center = center;
    }

    /**
     * Returns the Modules of the given name (search order is north-east-south-west-center).
     *
     * @param moduleName
     *            the name of the Module to extract from this Frontend.
     * @return an array of Modules of the given name. The array length may be 0, if no module of the
     *         given name is found.
     */
    public Module[] getModulesByName( String moduleName ) {

        List<Module> moduleList = new ArrayList<Module>( 5 );

        if ( getNorth() != null && getNorth().getModule( moduleName ) != null ) {
            moduleList.add( getNorth().getModule( moduleName ) );
        }
        if ( getEast() != null && getEast().getModule( moduleName ) != null ) {
            moduleList.add( getEast().getModule( moduleName ) );
        }
        if ( getSouth() != null && getSouth().getModule( moduleName ) != null ) {
            moduleList.add( getSouth().getModule( moduleName ) );
        }
        if ( getWest() != null && getWest().getModule( moduleName ) != null ) {
            moduleList.add( getWest().getModule( moduleName ) );
        }
        if ( getCenter() != null && getCenter().getModule( moduleName ) != null ) {
            moduleList.add( getCenter().getModule( moduleName ) );
        }
        return moduleList.toArray( new Module[moduleList.size()] );
    }

}
