//$HeadURL: https://svn.wald.intevation.org/svn/deegree/base/branches/2.3_testing/src/org/deegree/portal/standard/context/control/InitDownloadListener.java $$
/*----------------------------------------------------------------------------
 This file is part of deegree, http://deegree.org/
 Copyright (C) 2001-2009 by:
   Department of Geography, University of Bonn
 and
   lat/lon GmbH

 This library is free software; you can redistribute it and/or modify it under
 the terms of the GNU Lesser General Public License as published by the Free
 Software Foundation; either version 2.1 of the License, or (at your option)
 any later version.
 This library is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 details.
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

 Contact information:

 lat/lon GmbH
 Aennchenstr. 19, 53177 Bonn
 Germany
 http://lat-lon.de/

 Department of Geography, University of Bonn
 Prof. Dr. Klaus Greve
 Postfach 1147, 53001 Bonn
 Germany
 http://www.geographie.uni-bonn.de/deegree/

 e-mail: info@deegree.org
----------------------------------------------------------------------------*/

package org.deegree.portal.standard.context.control;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.deegree.enterprise.control.FormEvent;
import org.deegree.framework.log.ILogger;
import org.deegree.framework.log.LoggerFactory;
import org.deegree.portal.Constants;
import org.deegree.portal.context.DataService;
import org.deegree.portal.context.Layer;
import org.deegree.portal.context.LayerExtension;
import org.deegree.portal.context.ViewContext;
import org.deegree.security.drm.SecurityAccess;
import org.deegree.security.drm.SecurityAccessManager;
import org.deegree.security.drm.model.RightType;
import org.deegree.security.drm.model.SecuredObject;
import org.deegree.security.drm.model.User;

/**
 *
 *
 * @author <a href="mailto:poth@lat-lon.de">Andreas Poth </a>
 * @author last edited by: $Author: mschneider $
 *
 * @version 1.1, $Revision: 18195 $, $Date: 2009-06-18 17:55:39 +0200 (Do, 18. Jun 2009) $
 *
 * @since 1.1
 */
public class InitDownloadListener extends AbstractContextListener {

    private static final ILogger LOG = LoggerFactory.getLogger( InitDownloadListener.class );

    /*
     * (non-Javadoc)
     *
     * @see org.deegree.enterprise.control.AbstractListener#actionPerformed(org.deegree.enterprise.control.FormEvent)
     */
    @Override
    public void actionPerformed( FormEvent event ) {

        Map map = toModel();
        try {
            String userName = getUserName( (String) map.get( "SESSIONID" ) );
            String userPw = getUserPassword( (String) map.get( "SESSIONID" ) );
            Layer[] layers = getLayers( userName, userPw );
            getRequest().setAttribute( "LAYERS", layers );
        } catch ( Exception e ) {
            LOG.logError( e.getMessage(), e );
            gotoErrorPage( e.getMessage() );
            return;
        }
    }

    /**
     * returns true if the passed user is allowed to download the passed featureType
     *
     * @param user
     * @param featureType
     * @return true if the user is authorized, false otherwise
     */
    private boolean isAuthorized( String user, String password, String featureType ) {

        try {
            LOG.logDebug( "SecurityAccesManagr.isInitialized: ", SecurityAccessManager.isInitialized() );
            if ( !SecurityAccessManager.isInitialized() ) {
                initSAM();
            }
            SecurityAccessManager sam = SecurityAccessManager.getInstance();
            User usr = sam.getUserByName( user );
            usr.authenticate( password );
            SecurityAccess access = sam.acquireAccess( usr );
            SecuredObject secObj = access.getSecuredObjectByName( featureType, "Featuretype" );
            if ( !usr.hasRight( access, RightType.GETFEATURE, secObj ) ) {
                LOG.logError( "You are trying to access a feature/resource on a securedObject, which you do not have authentication to: "
                              + featureType );
                return false;
            } else {
                return true;
            }
        } catch ( Exception e ) {
            LOG.logError( e.getMessage(), e );
        }
        return false;
    }

    protected boolean initSAM() {

        String driver = getInitParameter( "driver" );
        String url = getInitParameter( "url" );
        String user = getInitParameter( "user" );
        String password = getInitParameter( "password" );

        Properties properties = new Properties();
        properties.setProperty( "driver", driver );
        properties.setProperty( "url", url );
        properties.setProperty( "user", user );
        properties.setProperty( "password", password );

        try {
            SecurityAccessManager.initialize( "org.deegree.security.drm.SQLRegistry", properties, 60 * 1000 );
        } catch ( Exception e ) {
            LOG.logError( "securitymanager could not be initilaized" );
            return false;
        }
        return false;
    }

    /**
     * returns a list of layers that are downloadable (assigend to a WFS) and for which the passed user has access
     * rights
     *
     * @param user
     *            username
     * @param pw
     *            password
     * @return all layers allowed for the given user
     */
    private Layer[] getLayers( String user, String pw ) {
        HttpSession session = ( (HttpServletRequest) this.getRequest() ).getSession();
        ViewContext vc = (ViewContext) session.getAttribute( Constants.CURRENTMAPCONTEXT );
        List<Layer> list = new ArrayList<Layer>();
        Layer[] layers = vc.getLayerList().getLayers();
        LOG.logDebug( "Calling getLayers()" );
        for ( int i = 0; i < layers.length; i++ ) {
            // gets the dataservice (WFS) that is responsible for
            // delivering a layers data
            DataService ds = ( (LayerExtension) layers[i].getExtension() ).getDataService();
            if ( ds != null ) {
                if ( isAuthorized( user, pw, ds.getFeatureType() ) ) {
                    list.add( layers[i] );
                    LOG.logDebug( "Adding layer: ", layers[i].getName(), " to list" );
                } else {
                    LOG.logDebug( "You are not authorized to Layer: ", layers[i].getName() );
                }
            } else {
                LOG.logDebug( "Dataservice for layer: ", layers[i].getName(), " is null" );
            }
        }
        layers = new Layer[list.size()];
        return (Layer[]) list.toArray( layers );
    }

}
