//$HeadURL: https://svn.wald.intevation.org/svn/deegree/base/branches/2.3_testing/src/org/deegree/portal/standard/sos/configuration/ConfigurationFactory.java $

/*----------------------------------------------------------------------------
 This file is part of deegree, http://deegree.org/
 Copyright (C) 2001-2009 by:
   Department of Geography, University of Bonn
 and
   lat/lon GmbH

 This library is free software; you can redistribute it and/or modify it under
 the terms of the GNU Lesser General Public License as published by the Free
 Software Foundation; either version 2.1 of the License, or (at your option)
 any later version.
 This library is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 details.
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

 Contact information:

 lat/lon GmbH
 Aennchenstr. 19, 53177 Bonn
 Germany
 http://lat-lon.de/

 Department of Geography, University of Bonn
 Prof. Dr. Klaus Greve
 Postfach 1147, 53001 Bonn
 Germany
 http://www.geographie.uni-bonn.de/deegree/

 e-mail: info@deegree.org
----------------------------------------------------------------------------*/
package org.deegree.portal.standard.sos.configuration;

import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.util.HashMap;

import org.deegree.framework.xml.ElementList;
import org.deegree.framework.xml.XMLTools;
import org.deegree.ogcbase.CommonNamespaces;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

/**
 * @author <a href="mailto:che@wupperverband.de.de">Christian Heier</a>
 * @author last edited by: $Author: mschneider $
 *
 * @version $Revision: 18195 $, $Date: 2009-06-18 17:55:39 +0200 (Do, 18. Jun 2009) $
 */
public class ConfigurationFactory {
    private static final URI SOSNS = CommonNamespaces.buildNSURI( "http://www.deegree.org/sosclient" );

    /**
     *
     *
     * @param confFile
     *
     * @return Configuration
     *
     * @throws SAXException
     * @throws IOException
     * @throws Exception
     */
    public static SOSClientConfiguration createConfiguration( String confFile )
                            throws SAXException, IOException, Exception {

        Reader reader = new FileReader( confFile );
        SOSClientConfiguration conf = createConfiguration( reader );
        reader.close();

        return conf;
    }

    /**
     *
     *
     * @param reader
     *
     * @return SOSClientConfiguration
     *
     * @throws SAXException
     * @throws IOException
     * @throws Exception
     */
    public static SOSClientConfiguration createConfiguration( Reader reader )
                            throws SAXException, IOException, Exception {

        Document doc = XMLTools.parse( reader );

        // sos descriptions
        Element element = doc.getDocumentElement();
        ElementList el = XMLTools.getChildElements( "sos", SOSNS, element );
        HashMap<String, URL> sosServers = createSOSDesc( el );

        return new SOSClientConfiguration( sosServers );
    }

    /**
     * creates a map of sos names and associated online resources
     */
    private static HashMap<String, URL> createSOSDesc( ElementList nl )
                            throws MalformedURLException {

        HashMap<String, URL> sosServers = new HashMap<String, URL>();

        for ( int i = 0; i < nl.getLength(); i++ ) {
            Element element = nl.item( i );
            Node node = element.getElementsByTagNameNS( SOSNS.toString(), "name" ).item( 0 );
            String name = node.getFirstChild().getNodeValue();
            node = element.getElementsByTagNameNS( SOSNS.toString(), "onlineResource" ).item( 0 );

            String tmp = XMLTools.getStringValue( node );
            sosServers.put( name, new URL( tmp ) );
        }

        return sosServers;
    }

}
