//$HeadURL: https://svn.wald.intevation.org/svn/deegree/base/branches/2.3_testing/src/org/deegree/portal/wac/WACServlet.java $
/*----------------------------------------------------------------------------
 This file is part of deegree, http://deegree.org/
 Copyright (C) 2001-2009 by:
   Department of Geography, University of Bonn
 and
   lat/lon GmbH

 This library is free software; you can redistribute it and/or modify it under
 the terms of the GNU Lesser General Public License as published by the Free
 Software Foundation; either version 2.1 of the License, or (at your option)
 any later version.
 This library is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 details.
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

 Contact information:

 lat/lon GmbH
 Aennchenstr. 19, 53177 Bonn
 Germany
 http://lat-lon.de/

 Department of Geography, University of Bonn
 Prof. Dr. Klaus Greve
 Postfach 1147, 53001 Bonn
 Germany
 http://www.geographie.uni-bonn.de/deegree/

 e-mail: info@deegree.org
----------------------------------------------------------------------------*/
package org.deegree.portal.wac;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.deegree.framework.log.ILogger;
import org.deegree.framework.log.LoggerFactory;
import org.deegree.framework.util.KVP2Map;
import org.deegree.framework.util.StringTools;
import org.deegree.framework.xml.XMLFragment;
import org.deegree.model.metadata.iso19115.Linkage;
import org.deegree.model.metadata.iso19115.OnlineResource;
import org.deegree.ogcwebservices.InvalidParameterValueException;
import org.deegree.ogcwebservices.wms.XMLFactory;
import org.deegree.ogcwebservices.wms.capabilities.WMSCapabilities;
import org.deegree.ogcwebservices.wms.capabilities.WMSCapabilitiesDocument;
import org.deegree.ogcwebservices.wms.capabilities.WMSCapabilitiesDocumentFactory;
import org.deegree.owscommon_new.DCP;
import org.deegree.owscommon_new.HTTP;
import org.deegree.owscommon_new.Operation;
import org.deegree.owscommon_new.OperationsMetadata;
import org.w3c.dom.Document;

/**
 *
 *
 * @author <a href="mailto:poth@lat-lon.de">Andreas Poth </a>
 * @author last edited by: $Author: mschneider $
 *
 * @version $Revision: 18195 $, $Date: 2009-06-18 17:55:39 +0200 (Do, 18. Jun 2009) $
 */
public class WACServlet extends HttpServlet {

    /**
     *
     */
    private static final long serialVersionUID = -4964542190148138040L;

    private Map<String, String> users = new HashMap<String, String>();

    private Map<String, String> passwords = new HashMap<String, String>();

    private Map<String, String> sessionIDs = Collections.synchronizedMap( new HashMap<String, String>() );

    private String host = null;

    private int port = 443;

    private String path = null;

    private String certificate = null;

    private String wacURL = null;

    private static final ILogger LOG = LoggerFactory.getLogger( WACServlet.class );

    @Override
    public void init()
                            throws ServletException {
        super.init();
        String user = getInitParameter( "USER" );
        if ( user == null ) {
            throw new ServletException( "user must be set!" );
        }
        users.put( "*", user );
        String password = getInitParameter( "PASSWORD" );
        if ( password == null ) {
            throw new ServletException( "password must be set!" );
        }
        passwords.put( "*", password );
        host = getInitParameter( "HOST" );
        if ( host == null ) {
            throw new ServletException( "WSS host must be set!" );
        }
        try {
            port = Integer.parseInt( getInitParameter( "PORT" ) );
        } catch ( NumberFormatException e ) {
            getServletContext().log( "-> using default SSL port 443" );
        }
        path = getInitParameter( "PATH" );
        if ( path == null ) {
            throw new ServletException( "path to web application on host must be set!" );
        }
        certificate = getInitParameter( "CERTIFICATE" );
        if ( certificate == null ) {
            getServletContext().log( "no certificate defined" );
        }

    }

    /**
     * @param request
     * @param response
     * @throws IOException
     */
    @Override
    protected void doGet( HttpServletRequest request, HttpServletResponse response )
                            throws IOException {

        wacURL = "http://" + request.getServerName() + ":" + request.getServerPort() + request.getContextPath()
                 + request.getServletPath();

        String user = users.get( "*" );
        WAClient wac = new WAClient( host, path, port, certificate );
        if ( sessionIDs.get( user ) == null ) {
            if ( !accessSessionID( wac, response ) ) {
                return;
            }
        }
        // get sessionID assigned to the user
        String sessionID = sessionIDs.get( user );
        InputStream is = null;
        try {
            StringBuffer sb = new StringBuffer( 2000 );
            sb.append( request.getQueryString() ).append( "&sessionID=" ).append( sessionID );
            is = wac.performDoService( request.getQueryString(), sessionID );
        } catch ( Exception e ) {
            e.printStackTrace();
            sendException( response, "GetSession", "could not perform DoService", StringTools.stackTraceToString( e ) );
        }

        OutputStream os = null;
        try {
            os = response.getOutputStream();
            postProcess( request.getQueryString(), is, os );
        } catch ( Exception e ) {
            sendException( response, "GetSession", "could not post process capabilities",
                           StringTools.stackTraceToString( e ) );
        } finally {
            os.flush();
            os.close();
            is.close();
        }
    }

    /**
     * access a sessionID from a WSS and stores it into an internal Map to use for DoService calls
     * against a WSS
     *
     * @param wac
     * @param response
     * @return true if a valid session id could be retrieved.
     */
    private boolean accessSessionID( WAClient wac, HttpServletResponse response ) {
        String user = users.get( "*" );
        String password = passwords.get( "*" );
        try {
            String sessionID = wac.performGetSession( user, password );
            sessionIDs.put( user, sessionID );
        } catch ( WACException e ) {
            e.printStackTrace();
            sendException( response, "GetSession", "could not perform GetSession", StringTools.stackTraceToString( e ) );
            return false;
        } catch ( Exception e ) {
            e.printStackTrace();
            sendException( response, "GetSession", "could not evaluate GetSession result",
                           StringTools.stackTraceToString( e ) );
            return false;
        }
        return true;
    }

    private void sendException( HttpServletResponse response, String req, String message, String stacktrace ) {
        this.getServletContext().log( message );
        this.getServletContext().log( stacktrace );
        response.setContentType( "text/xml" );
        if ( req == null )
            req = "";
        if ( message == null )
            message = "";
        try {
            PrintWriter pw = response.getWriter();
            pw.write( "<OGCWebServiceException>" );
            pw.write( "<Message>" );
            pw.write( req );
            pw.write( ": failed! " );
            pw.write( message );
            pw.write( "</Message>" );
            pw.write( "<Locator>" );
            pw.write( stacktrace );
            pw.write( "</Locator>" );
            pw.write( "</OGCWebServiceException>" );
            pw.close();
        } catch ( Exception ee ) {
            ee.printStackTrace();
        }
    }

    /**
     * forces a post processing of the wss response if a GetCapabilities request has been performed
     * by replacing contained the online resources
     *
     * @param request
     * @param is
     * @param os
     *            stream to write the result too
     * @throws Exception
     */
    private void postProcess( String request, InputStream is, OutputStream os )
                            throws Exception {
        Map<String, String> map = KVP2Map.toMap( request );
        if ( map.get( "REQUEST" ).equals( "GetCapabilities" ) ) {
            XMLFragment xml = new XMLFragment();
            xml.load( is, XMLFragment.DEFAULT_URL );
            is.close();
            Document doc = xml.getRootElement().getOwnerDocument();
            if ( map.get( "SERVICE" ).equals( "WMS" ) ) {
                adjustWMSCapabilities( doc, os );
            } else if ( map.get( "SERVICE" ).equals( "WFS" ) ) {
                // TODO
            } else if ( map.get( "SERVICE" ).equals( "WCS" ) ) {
                // TODO
            } else if ( map.get( "SERVICE" ).equals( "CSW" ) ) {
                // TODO
            } else if ( map.get( "SERVICE" ).equals( "SCS" ) ) {
                // TODO
            }
            if ( map.get( "SERVICE" ).equals( "WFS-G" ) ) {
                // TODO
            }
            if ( map.get( "SERVICE" ).equals( "WTS" ) ) {
                // TODO
            }
        } else {
            byte[] b = new byte[1024];
            int c = 0;
            while ( ( c = is.read( b ) ) > 0 ) {
                os.write( b, 0, c );
            }
            os.close();
            is.close();
        }
    }

    /**
     * adjusts the passed WMS capabilities document by replacing the contained online resources with
     * the address of the WAC
     * @param doc
     * @param os
     * @throws InvalidParameterValueException
     */
    private void adjustWMSCapabilities( Document doc, OutputStream os )
                            throws InvalidParameterValueException, IOException {

        WMSCapabilities capa = null;
        try {
            WMSCapabilitiesDocument cdoc = WMSCapabilitiesDocumentFactory.getWMSCapabilitiesDocument( doc.getDocumentElement() );
            capa = (WMSCapabilities) cdoc.parseCapabilities();
        } catch ( Exception e ) {
            e.printStackTrace();
            throw new InvalidParameterValueException( "no valid wms capabilities\n"
                                                      + StringTools.stackTraceToString( e ) );
        }

        OperationsMetadata om = capa.getOperationMetadata();

        List<Operation> ops = om.getOperations();
        for ( Operation operation : ops ) {
            setNewOnlineResource( operation );
        }

        WMSCapabilitiesDocument cdoc = XMLFactory.export( capa );
        cdoc.write( os );
    }

    /**
     * sets a new online resource for the passed <tt>Operation</tt>
     *
     * @param op
     */
    private void setNewOnlineResource( Operation op ) {
        List<DCP> dcps = op.getDCP();
        if ( dcps != null ) {
            for ( DCP dcp : dcps ) {
                HTTP http = (HTTP) dcp;
                try {
                    URL url = new URL( wacURL );
                    OnlineResource link = new OnlineResource( new Linkage( url ) );
                    List<OnlineResource> resources = http.getLinks();
                    int size = resources.size();
                    resources.clear();
                    for ( int i = 0; i < size; ++i )
                        resources.add( link );
                } catch ( MalformedURLException e1 ) {
                    LOG.logError( e1.getLocalizedMessage(), e1 );
                }
            }
        }
    }

    /**
     * @param request
     * @param response
     * @throws ServletException
     * @throws IOException
     */
    @Override
    protected void doPost( HttpServletRequest request, HttpServletResponse response )
                            throws ServletException, IOException {
        doGet( request, response );
    }

}
